package javax.portlet;

import java.io.IOException;
import java.lang.IllegalStateException;
import java.lang.annotation.*;
import java.lang.reflect.*;
import java.util.*;

public abstract class GenericPortlet implements Portlet, EventPortlet, PortletConfig, ResourceServingPortlet {

  private transient PortletConfig config;

  public GenericPortlet() {}

  public void init(PortletConfig config) throws PortletException {
    this.config = config;
    this.init();
  }

  public void init() throws PortletException {
  }

  public void processEvent(EventRequest eventrequest, EventResponse eventresponse) throws PortletException, IOException {
    Method[] mme = this.getClass().getMethods();
    for (Method me : mme) {
      Annotation an = me.getAnnotation(ProcessEvent.class);
      if (an != null) {
        if (!((ProcessEvent) an).name().equals(eventrequest.getEvent().getName().toString()))
          continue;
        try {
          me.invoke(this, eventrequest, eventresponse);
        } catch (Exception e) {
          throw new PortletException("processEvent method can't invoke annotated event processing method " +
            this.getClass().getName() + "." + me.getName(), e);
        }
      }
    }
  }

  public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
    throw new PortletException("processAction method not implemented");
  }

  public void render(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    if (request.getAttribute(RenderRequest.RENDER_PART) != null) {  // streaming portal calling
      if (request.getAttribute(RenderRequest.RENDER_PART).equals(RenderRequest.RENDER_HEADERS) ) {
          doHeaders(request, response);
          Enumeration nextModes = getNextPossiblePortletModes(request);
          if (nextModes != null)
            response.setNextPossiblePortletModes(nextModes);
          response.setTitle(getTitle(request));
      } else if (request.getAttribute(RenderRequest.RENDER_PART).equals(RenderRequest.RENDER_MARKUP) ) {
          doDispatch(request, response);
      }
    } else {  // buffered portal calling
      doHeaders(request, response);
      response.setTitle(getTitle(request));
      doDispatch(request, response);
    }
  }

  protected String getTitle(RenderRequest request) {
    if (config == null)
      throw new IllegalStateException("Config is null, please ensure that your init(config) method calls super.init(config)");

    return config.getResourceBundle(request.getLocale()).getString("javax.portlet.title");
  }

  protected void doDispatch(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    WindowState state = request.getWindowState();
    if (!state.equals(WindowState.MINIMIZED)) {
      PortletMode mode = request.getPortletMode();
      if (mode.equals(PortletMode.VIEW)) {
        doView(request, response);
      } else
      if (mode.equals(PortletMode.EDIT)) {
        doEdit(request, response);
      } else
      if (mode.equals(PortletMode.HELP)) {
        doHelp(request, response);
      } else {
        throw new PortletException("unknown portlet mode: " + mode);
      }
    }
  }

  protected void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    throw new PortletException("doView method not implemented");
  }

  protected void doEdit(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    throw new PortletException("doEdit method not implemented");
  }

  protected void doHelp(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    throw new PortletException("doHelp method not implemented");
  }

  public void serveResource(ResourceRequest resourcerequest, ResourceResponse resourceresponse) throws PortletException, IOException {
    if ( resourcerequest.getResourceID() != null ) {
      PortletRequestDispatcher rd = getPortletConfig().getPortletContext().getRequestDispatcher(resourcerequest.getResourceID());
      rd.forward(resourcerequest, resourceresponse);
    }
  }

  public PortletConfig getPortletConfig() {
    return config;
  }

  public void destroy() {
  }

  public String getPortletName() {
    return config.getPortletName();
  }

  public PortletContext getPortletContext() {
    return config.getPortletContext();
  }

  public ResourceBundle getResourceBundle(Locale locale) {
    return config.getResourceBundle(locale);
  }

  public String getInitParameter(String name) {
    return config.getInitParameter(name);
  }

  public Enumeration getInitParameterNames() {
    return config.getInitParameterNames();
  }

  protected void doHeaders(RenderRequest request, RenderResponse response) {
    return;
  }

  protected Enumeration getNextPossiblePortletModes(RenderRequest request) {
    return null;
  }
  
  public Enumeration getPublicRenderParameterNames() {
    return null;
  }
  
}
